@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import datetime
+import itertools as itt
 import json
 import re
 import traceback
@@ -14,8 +15,8 @@ from . import FluxCog
 from .. import CommandEvent, utils
 from ..auth import Auth, AuthList, Record
 from ..command import Response
-from ..context import ManualAuthCtx, ManualAuthorCtx, CommandCtx, GuildMessageCtx
-from ..errors import CommandError, BotMissingPermissions
+from ..context import CommandCtx, GuildMessageCtx, ManualAuthCtx, ManualAuthorCtx
+from ..errors import CommandError
 
 if ty.TYPE_CHECKING:
    from ..context import GuildAwareCtx
@@ -40,30 +41,33 @@ class Builtins(FluxCog):
 
             if not ids_:
                raise CommandError(f"No member ID found in {target_}")
-            if not await utils.get_or_fetch_member(ctx.guild, ids_[0]):
+            if not await self.flux.get_member_s(ctx.guild, ids_[0]):
                raise CommandError(f"No member found with ID {ids_[0]}")
             return ids_[0]
          if type_ == "role":
             ids_ = utils.find_mentions(target_)
-            try:
-               role_id = ctx.guild.get_role(int(ids_[0])).id
-            except AttributeError:
+            role = ctx.guild.get_role(ids_[0])
+            if role is None:
                raise CommandError(f"No role found with ID {target_}")
-            return role_id
+            return role.id
          if type_ == "permissions":
             p_dict: ty.List[str] = json.loads(target_)
             return discord.Permissions(**{p: True for p in p_dict}).value
+         else:
+            raise ValueError
 
       async def parse_auth_context(ctx: GuildAwareCtx, type_: str, target_: str) -> ManualAuthCtx:
          if type_ == "user":
             ids_ = utils.find_mentions(target_)
             if not ids_:
                raise CommandError(f"No user ID found in {target_}")
-
-            return ManualAuthCtx(flux=self.flux, auth_list=AuthList(user=self.flux.get_user(ids_[0]).id), config_identifier=ids_[0])
+            user = await self.flux.get_user_s(ids_[0])
+            if not user:
+               raise CommandError(f"No user found with ID: {ids_[0]}")
+            return ManualAuthCtx(flux=self.flux, auth_list=AuthList(user=user.id), config_identifier=ids_[0])
          auth_id = await parse_auth_id(ctx, type_=type_, target_=target_)
          if type_ == "member":
-            member = await utils.get_or_fetch_member(ctx.guild, auth_id)
+            member = await self.flux.get_member_s(ctx.guild, auth_id)
             if not member:
                raise CommandError(f"No member found with id `{auth_id}`")
             return ManualAuthCtx(flux=self.flux,
@@ -88,21 +92,7 @@ class Builtins(FluxCog):
                   config_identifier=str(ctx.guild.id))
             except TypeError as e:
                raise CommandError(f"Permissions `{auth_id}` could not be parsed. See:\n{e}")
-
-      # @CommandCheck.check(lambda ctx: ctx.author.id == self.flux.admin_id)
-      # @self._commandeer(name="reload", private=True)
-      # async def reload(ctx: MessageContext, cog_name: str):
-      #    reloaded_cogs = []
-      #    for cog in s.cogs:
-      #       new_cog = cog
-      #       if cog.__class__.__name__.lower() == cog_name:
-      #          cog.teardown()
-      #          module = importlib.reload(inspect.getmodule(cog))
-      #          new_cog = getattr(module, cog.__class__.__name__)(ctx.flux)
-      #          await new_cog.startup()
-      #       reloaded_cogs.append(new_cog)
-      #    ctx.flux.cogs = reloaded_cogs
-      #    return Response()
+         raise ValueError
 
       @self._commandeer(name="asif", default_auths=[Record.allow_all()])
       async def __asif(ctx: GuildCommandCtx, args: str, ):
@@ -133,6 +123,7 @@ class Builtins(FluxCog):
             logger.info(e)
             raise CommandError(f"See `help asif` for usage")
 
+         # noinspection PyPep8Naming
          MOCK_TYPES = {
             "u"   : "user",
             "user": "user",
@@ -145,8 +136,10 @@ class Builtins(FluxCog):
 
          cmd_name, cmd_args, *_ = [*command.split(" ", 1), None]
          mock_auth_ctx = await parse_auth_context(ctx=ctx.msg_ctx, type_=mock_type, target_=mock_target)
-         if ctx.msg_ctx.message.guild:
-            mock_author_ctx = ManualAuthorCtx(author=await utils.get_or_fetch_member(ctx.msg_ctx.guild, mock_target))
+         if mock_type == "user":
+            mock_author_ctx = ManualAuthorCtx(author=await self.flux.fetch_user(int(mock_target)))
+         elif mock_type == "member" and ctx.msg_ctx.message.guild:
+            mock_author_ctx = ManualAuthorCtx(author=await self.flux.get_member_s(ctx.msg_ctx.guild, int(mock_target)))
          else:
             mock_author_ctx = ctx.author_ctx
 
@@ -245,7 +238,7 @@ class Builtins(FluxCog):
                auth_str,
                [x - 1 for x in [1, 2, 3, 4]]
             )
-         except (ValueError, AttributeError):
+         except (ValueError, AttributeError, TypeError):
             raise CommandError(f"See `help auth` for usage")
 
          rule_subject = rule_subject.lower()
@@ -289,7 +282,7 @@ class Builtins(FluxCog):
                raise CommandError(f"Cannot find a user/member in `{target_raw}`. It should either be an ID or a mention")
 
             if isinstance(ctx.msg_ctx, GuildMessageCtx):
-               target = await utils.get_or_fetch_member(ctx.msg_ctx.guild, target)
+               target = await self.flux.get_member_s(ctx.msg_ctx.guild, target)
             else:
                target = await ctx.msg_ctx.flux.get_user(target)
 
@@ -382,7 +375,7 @@ class Builtins(FluxCog):
                tabular_data=[[str(emoji), emoji.animated, emoji.url] for emoji in g.emojis]
             )
          )
-         embed.add_field(name="Emoji", value=f"[{len(g.emojis)}/{g.emoji_limit*2} emojis]({emoji_haste})", inline=False)
+         embed.add_field(name="Emoji", value=f"[{len(g.emojis)}/{g.emoji_limit * 2} emojis]({emoji_haste})", inline=False)
 
          # Features
          features_haste = await utils.haste(
@@ -435,7 +428,7 @@ class Builtins(FluxCog):
       @self._commandeer(name="help", default_auths=[Record.allow_all()])
       async def __get_help(ctx: GuildCommandCtx, help_target: ty.Optional[str]):
          """
-         help (command_name)
+         help (command_name/section)
          ==
          My brother is dying! Get Help!
          ==
@@ -447,17 +440,33 @@ class Builtins(FluxCog):
          """
          utils.perm_check(ctx.msg_ctx.channel, discord.Permissions(embed_links=True))
          configs = self.flux.CONFIG.of(ctx.msg_ctx)
-         authorized_cmds: ty.Dict[str, Command] = {command.name: command for cog in self.flux.cogs for command in cog.commands if
-                                                   Auth.accepts_all(ctx.auth_ctxs, command) and command.name != "help"}
-
+         authorized_cmds: ty.Dict[str, ty.Tuple[FluxCog, Command]] = {command.name: (cog, command) for cog in self.flux.cogs for command in cog.commands if
+                                                                      Auth.accepts_all(ctx.auth_ctxs, command) and command.name != "help"}
+         # Help
          if not help_target:
-            help_embed = discord.Embed(title=f"{utils.EMOJI.question} Command Help", description=f"{configs['prefix']}help <command> for more info")
-            for cmd_name, command in authorized_cmds.items():
-               usage = "\n".join([f"{configs['prefix']}{usage}" for usage in command.short_usage.split("\n")])
-               help_embed.add_field(name=cmd_name, value=usage, inline=False)
-
+            help_embed = discord.Embed(title=f"{utils.EMOJI.question} Help", description=f"{configs['prefix']}help <command/section> for more info")
+            cog_groups = itt.groupby(authorized_cmds.items(), lambda x: x[1][0])
+            for cog, group in cog_groups:
+               # noinspection Mypy
+               cog: FluxCog  # https://youtrack.jetbrains.com/issue/PY-43664
+               group: ty.Iterable[ty.Tuple[str, ty.Tuple[FluxCog, Command]]]
+               usages = "\n".join(
+                  ["\n".join([f"{configs['prefix']}{usage}" for usage in cmd_item[1][1].usage.split("\n")])
+                   for cmd_item in group]
+               )
+               help_embed.add_field(name=cog.name, value=usages, inline=False)
             return Response(embed=help_embed)
 
+         # Help for Cog
+
+         if cog := next((c for c in self.flux.cogs if c.name.lower() == help_target.lower()), None):
+            embed = discord.Embed(title=f"{utils.EMOJI.question} Section Help: {cog.name}")
+            for cmd in cog.commands:
+               if cmd.name in authorized_cmds:
+                  embed.add_field(name=cmd.name, value=cmd.description, inline=False)
+            return Response(embed=embed)
+
+         # Help for Help
          if help_target == "help":
             embed = discord.Embed(title="\U00002754 Command Help", description="How to read help")
             embed.add_field(name="Usage", value='..commandname [lit] <user> {json} (optional) extra*', inline=False)
@@ -471,24 +480,18 @@ class Builtins(FluxCog):
             return Response(embed=embed)
 
          if help_target not in authorized_cmds:
-            return Response(f"No command `{help_target}` to show help for", status="error")
+            return Response(f"No command or section `{help_target}` to show help for", status="error")
 
-         cmd = authorized_cmds[help_target]
+         # Help for Command
+         cog, cmd = authorized_cmds[help_target]
          embed = discord.Embed(
-            title=f"\U00002754 Command Help for {help_target}",
+            title=f"\U00002754 Command Help: {help_target}",
             description=cmd.description)
 
-         embed.add_field(name="Usage", value=cmd.short_usage, inline=False)
-         print(cmd.param_usage)
+         embed.add_field(name="Usage", value=cmd.usage, inline=False)
          for arg, detail in cmd.param_usage:
             embed.add_field(name=arg.strip(), value=detail.strip(), inline=False)
-            # embed.add_field(name=detail,value="", inline=False)
 
-         # args, details = list(zip(*cmd.long_usage))
-         # embed.add_field(name="Param", value="\n".join(args), inline=True)
-         # embed.add_field(name="Details", value="\n".join(details), inline=True)
-
-         # embed.add_field(name="usage", value=f"{configs['prefix']}{cmd.long_usage}", inline=False)
          return Response(embed=embed)
 
    @property
